<?php
/**
 * Initialize the custom theme options.
 */
add_action( 'init', 'custom_theme_options' );

/**
 * Build the custom settings & update OptionTree.
 */
function custom_theme_options() {
  
  /* OptionTree is not loaded yet, or this is not an admin request */
  if ( ! function_exists( 'ot_settings_id' ) || ! is_admin() )
    return false;
    
  /**
   * Get a copy of the saved settings array. 
   */
  $saved_settings = get_option( ot_settings_id(), array() );
  
  /**
   * Custom settings array that will eventually be 
   * passes to the OptionTree Settings API Class.
   */
  $custom_settings = array( 
    'contextual_help' => array( 
      'content'       => array( 
        array(
          'id'        => 'general_help',
          'title'     => 'General',
          'content'   => '<p>Help content goes here!</p>'
        )
      ),
      'sidebar'       => '<p>Sidebar content goes here!</p>'
    ),
    'sections'        => array( 
      array(
        'id'          => 'general',
        'title'       => 'General'
      )
    ),
    'settings'        => array( 
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_logo',
        'label'       => '',
        'desc'        => 'You can use your own logo. Click to \'Upload\' button and upload your own logo.',
        'std'         => 'http://lawnew.dev/static/evo/admin/images/loginlogo.png',
        'type'        => 'upload',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_logo_text',
        'label'       => '',
        'desc'        => 'If you don\'t upload your own company logo, this text will show up in it\'s place.',
        'std'         => 'Fizz',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_favicon',
        'label'       => '',
        'desc'        => 'You can use your own custom favicon. Click to \'Upload Image\' button and upload your own favicon.',
        'std'         => '',
        'type'        => 'upload',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_cta_display',
        'label'       => '',
        'desc'        => 'Display call-to-action section, above the footer',
        'std'         => '',
        'type'        => 'checkbox',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => 'tiny',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_cta_text',
        'label'       => '',
        'desc'        => '',
        'std'         => 'A call-to-action text',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_cta_button_text',
        'label'       => '',
        'desc'        => '',
        'std'         => 'Contact us',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_cta_button_link',
        'label'       => '',
        'desc'        => '',
        'std'         => '/contact/',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_customfontsinfo',
        'label'       => '',
        'desc'        => 'This theme uses Google Web Font for headings. You can change it by entering the font details in the fields below.',
        'std'         => '',
        'type'        => 'info',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_customtypography',
        'label'       => '',
        'desc'        => 'By unchecking this the theme will use default font for headings, Arial.',
        'std'         => '0',
        'type'        => 'checkbox',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_headingfontlink',
        'label'       => '',
        'desc'        => 'Ex: &lt;link href=\'http://fonts.googleapis.com/css?family=Sanchez\' rel=\'stylesheet\' type=\'text/css\'&gt;',
        'std'         => '',
        'type'        => 'textarea',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_headingfontface',
        'label'       => '',
        'desc'        => 'Ex: font-family: \'Sanchez\', serif',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_bodyfontlink',
        'label'       => '',
        'desc'        => 'Ex: &lt;link href=\'http://fonts.googleapis.com/css?family=Sanchez\' rel=\'stylesheet\' type=\'text/css\'&gt;',
        'std'         => '',
        'type'        => 'textarea',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_bodyfontface',
        'label'       => '',
        'desc'        => 'Ex: font-family: \'Sanchez\', serif',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_displayslider',
        'label'       => '',
        'desc'        => 'Display slider on homepage',
        'std'         => '',
        'type'        => 'checkbox',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => 'tiny',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_slidertag',
        'label'       => '',
        'desc'        => 'Choose your slider. If none selected, it will display all slides',
        'std'         => '',
        'type'        => 'select',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => 'tiny',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_slidertype',
        'label'       => '',
        'desc'        => 'Choose slider type for homepage',
        'std'         => 'flex',
        'type'        => 'select',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => 'tiny',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_slidereffect',
        'label'       => '',
        'desc'        => 'Default is \'slide\'',
        'std'         => 'slide',
        'type'        => 'select',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => 'tiny',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_slideranimationspeed',
        'label'       => '',
        'desc'        => 'Default is 500',
        'std'         => '500',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_sliderpausetime',
        'label'       => '',
        'desc'        => 'Default is 3000',
        'std'         => '3000',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_portfoliodesc',
        'label'       => '',
        'desc'        => 'The portfolio item page will use the same title description, if defined, as the selected page.',
        'std'         => '',
        'type'        => 'select',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_portfolioitemsperpage',
        'label'       => '',
        'desc'        => 'Set the number of items that appear on the Portfolio page.',
        'std'         => '8',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_singledesc',
        'label'       => '',
        'desc'        => 'The blog item page will use the same title description, if defined, as the selected page.',
        'std'         => '',
        'type'        => 'select',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_contact_email',
        'label'       => '',
        'desc'        => '',
        'std'         => 'info@yoursite.com',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_contact_phone',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_contact_fax',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_contact_address',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_contact_map',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'textarea',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_display_social',
        'label'       => '',
        'desc'        => '',
        'std'         => '1',
        'type'        => 'checkbox',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_facebook',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_twitter',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_dribble',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_linkedin',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_pinterest',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_rss',
        'label'       => '',
        'desc'        => 'Display RSS link',
        'std'         => '1',
        'type'        => 'checkbox',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_extrss',
        'label'       => '',
        'desc'        => 'Add external RSS URL, like Feedburner, etc. This will overwrite the regular blog RSS, if enabled.',
        'std'         => '',
        'type'        => 'text',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_metadescription',
        'label'       => '',
        'desc'        => '',
        'std'         => 'full functionable, premium resume wordpress theme solution for your website.',
        'type'        => 'textarea',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_metakeywords',
        'label'       => '',
        'desc'        => '',
        'std'         => 'proffesional wordpress theme, flexible wordpress theme, wordpress all in one theme, premium wordpress theme ',
        'type'        => 'textarea',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_footer_copyright',
        'label'       => '',
        'desc'        => 'You can change the footer copyright area.',
        'std'         => 'You can edit this copyright text from Theme Options > Footer',
        'type'        => 'textarea',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => 's5s_stats',
        'label'       => '',
        'desc'        => 'You can use google analytics or other stats code in this area it will appear in the source.',
        'std'         => '',
        'type'        => 'textarea',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => '',
        'std'         => '',
        'type'        => 'heading',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      ),
      array(
        'id'          => '',
        'label'       => '',
        'desc'        => 'Click on <a href=\'http://lawnew.dev/home/wp-admin/admin.php?page=ajax-thumbnail-rebuild\'>this link</a> to regenerate your existing thumbnails to fit the theme.',
        'std'         => '',
        'type'        => 'info',
        'section'     => 'general',
        'rows'        => '',
        'post_type'   => '',
        'taxonomy'    => '',
        'min_max_step'=> '',
        'class'       => '',
        'condition'   => '',
        'operator'    => ''
      )
    )
  );
  
  /* allow settings to be filtered before saving */
  $custom_settings = apply_filters( ot_settings_id() . '_args', $custom_settings );
  
  /* settings are not the same update the DB */
  if ( $saved_settings !== $custom_settings ) {
    update_option( ot_settings_id(), $custom_settings ); 
  }
  
  /* Lets OptionTree know the UI Builder is being overridden */
  global $ot_has_custom_theme_options;
  $ot_has_custom_theme_options = true;
  
}